# ************************************************************************
# * Copyright (c) infrae in 2002
# * For license details please see the license.txt file. 
# *
# * Project: DOC to Silva XML Converter
# *
# * Creation Date: June 2002
# *
# * Authors: Stephan Richter
# ************************************************************************
"""

$Id: Heading.py,v 1.4 2003/06/19 16:01:26 guido Exp $
"""

# XXX I duplicated a lot of code from P.py because the markup
# processing code was an intricate part of the P class and
# (at that moment) impossible to use in a more elegant way
# It would be nice if P.py was refactored at some point so
# the duplicated code can go here

from SilvaObject import SilvaObject
from util import replaceMap, stripControlChars
from silva2word.WordConstants import *
import re
from P import Tag

p_INDEX = re.compile(r' "([^"]+)" ')

class Heading(SilvaObject):

    __handled_fields = [wdFieldIndexEntry, wdFieldHyperlink]

    def __init__(self, original, type=None):
        SilvaObject.__init__(self, original)
        self.text = u''
        self.type = type
        if self.type is None:
            self.type = 'normal'
        self.markup = []
        self._non_indexes = [] 
        self._non_indexes_sorted = True
        self.handleHeading(original)

    def handleHeading(self, wp, handle_styles=1, correct_first=0):
        #self.debugfp.write('\nGoing to handle %s\n' % str(wp).encode('ascii', 'replace'))
        #self.debugfp.write('Handle styles: %s\n' % (not not handle_styles))
       
        def merge(list1, list2):
            """merge two insertions lists, preserving sorting""" 
            sort_dict = {}
            for tag in list1+list2:
                sort_dict.setdefault(tag.position, []).append(tag)
            indexes = sort_dict.keys()
            indexes.sort()
            result = []
            for index in indexes:
                for tag in sort_dict[index]:
                    result.append(tag)
            return result                    
        
        if wp is None: 
            return
        
        text = wp.Range()
        wp_start = wp.Range.Start
        wp_end = wp.Range.End

        #self.debugfp.write('Text from paragraph: %s\n\n' % text)
        #self.debugfp.write('Text start: %s\n' % wp_start)
        #self.debugfp.write('Text end: %s\n' % wp_end)
        
        if stripControlChars(text.strip()) == '':
            return
        xml = u''
        inserts = []
        
        """
        if correct_first:
            # somehow the first paragraph of each document needs 1 char more 
            # room than the rest
            self.nonIndex(wp_start, wp_start + 1)
        """
        
        #if handle_styles:
        #    #self.debugfp.write('Going to handle styles\n')
        #    inserts = self.processStyles(wp)
        #    #self.debugfp.write('Going to handle indexes\n')
        inserts = merge(inserts, self.processIndexes(wp))
        #    #self.debugfp.write('Going to handle links\n')
        #    inserts = merge(inserts, self.processLinks(wp))

        # well forming shouldn't be necessary since we don't allow markup
        # inserts = self.wellform(inserts)
        
        #self.debugfp.write('Inserts: %s\n' % inserts)
        self._blank_indexes(wp)
        self.setupKitchen()
        for tag in inserts:
            if tag.position < wp_start:
                #self.debugfp.write('Start position corrected\n')
                tag.position = wp_start
            elif tag.position > wp_end:
                #self.debugfp.write('End position corrected\n')
                tag.position = wp_end
            tag.position -= wp_start
            tp = tag.position
            tag.position = self.cookIndex(tag.position)
            #self.debugfp.write('Before cookIndex: %s, after: %s\n' % (tp, tag.position))
            #if correct_first:
            #    #self.debugfp.write('Correcting first item\n')
            #    tag.position -= 1

        next = 0
        for tag in inserts:
            #self.debugfp.write('Tag: %s, pos: %s\n' % (tag, tag.position))
            xml += replaceMap(text[next:tag.position])
            xml += tag.tag()
            next = tag.position
            #self.debugfp.write('XML so far: %s\n' % xml)

        xml += replaceMap(text[next:])
        xml = stripControlChars(xml)
                
        self.text = xml

    def handleText(self, wp):
        self.text = ' '.join(wp.Range().split())
        self.text = replaceMap(stripControlChars(self.text))

    def getXML(self):
        xml = u'<heading type="%s">' % (self.type, )
        xml += self.text
        xml += u'</heading>'
        return xml
            
    def processIndexes(self, wp):
        #debugfp = open(r'c:\index_debug', 'a')
        #self.debugfp.write('processIndexes\n')
            
        if len(wp.Range.Fields) == 0:
            #debugfp.write('No fields\n')
            return []
    
        wp_start = wp.Range.Start
        #debugfp.write('Wp start: %s\n' % wp_start)
        result = []
        for field in wp.Range.Fields:
            if not field.type == wdFieldIndexEntry: 
                #debugfp.write('No fieldindexentry\n')
                continue 
            
            start = field.Code.Start
            end = field.Code.End

            #debugfp.write('Start of code: %s\n' % start)
            #debugfp.write('End of code: %s\n' % end)
            #debugfp.write('End - 2: %s\n' % wp.Range.Document.Range(end - 2, end - 2)())
            #debugfp.write('Field content: %s\n' % field.Code())
            #debugfp.write('Field content: %s\n' % wp.Range.Document.Range(start, end))
            
            # somehow this needs 1 character extra space
            self.nonIndex(start, end + 1)
            
            m = p_INDEX.search(field.Code())
            if m is None: 
                #debugfp.write('No index\n')
                continue
            name = m.group(1)
            #debugfp.write('Added name %s, loc %s\n' % (name, end))
            
            tag = Tag('index', 'name="%s"' % (replaceMap(name), ), None, end)
            result.append(tag)
   
        return result

    def nonIndex(self, start, end):

        self._non_indexes += range(start - 1, end)
        self._non_indexes_sorted = False
   
    def setupKitchen(self):
        """sets up the kitchen for cooking indexes"""
        
        wp_start = self.original.Range.Start
        wp_end = self.original.Range.End
        if not self._non_indexes_sorted:
            self._non_indexes = dict(map(lambda x: (x, 0), 
                self._non_indexes)).keys()
            self._non_indexes.sort()
            self._non_indexes = \
                map(lambda x: x - wp_start, 
                filter(lambda x: wp_start <= x <= wp_end,
                    self._non_indexes))
            self._non_indexes_sorted = True
   
    def cookIndex(self, raw_index):
        #self.debugfp.write('Going to cook index for %s\n' % raw_index)
        nis = self._non_indexes
        #self.debugfp.write('Nis: %s\n' % nis)
        
        while raw_index in nis:
            raw_index += 1
        corrector = 0
        for i in range(0, len(nis)):
            if raw_index <= nis[i]:
                break
            else:
                corrector += 1
        #self.debugfp.write('Corrected index: %s\n' % (raw_index - corrector))
        return raw_index - corrector

    def _blank_indexes(self, wp):
        if len(wp.Range.Fields) == 0: 
            return
        for field in wp.Range.Fields:
            if not field.Type in self.__handled_fields:
                self.nonIndex(field.Code.Start, field.Code.End)

